#pragma once

#include <vector>

struct InstCfg;

class RootFilter
{
public:
    RootFilter();
    virtual ~RootFilter();
    void init();

protected:
    // interface to mpxmanager is formed by series of static member functions
    static int filterFunc(INTPTR userData, byte *in, byte *out);
    static int createInstFunc(INTPTR userData, ITEMID *instID);
    static int deleteInstFunc(INTPTR userData, ITEMID instID);
    static int setInstParFunc(INTPTR userData, ITEMID instID, int idx, byte *data, int dataSize);
    static int getInstParFunc(INTPTR userData, ITEMID instID, int idx, ItemInfo *info, int *dataSize);
    

    int filter(FRAMEID frameID, ITEMID instID);
    int createInst(ITEMID *instID);
    int deleteInst(ITEMID instID);
    int setInstPar(ITEMID instID, int idx, byte *data, int dataSize);
    int getInstPar(ITEMID instID, int idx, ItemInfo *info, int *dataSize);
    template<typename T> int save(FRAMEID frameID, const char *fileName, T *buff, u32 width, u32 height);

private:
    std::vector<InstCfg*> instCfgs;     // here we will store informations about instance configurations    
    CRITICAL_SECTION critSection;       // filter can be called simultaneously from different thread, we need object for synchronization
    static const int parCount = 1;      // number of parameters that have this filter
    static ItemInfo parDescr[parCount]; // parameter description
};

// structure holding information specific to instance (configuration)
struct InstCfg
{
    InstCfg()
        : filePathName(".\\files\\frame.root")
    {}
    std::string filePathName;
};

class LockCritSect
{
public:
    LockCritSect(CRITICAL_SECTION *_cs)
        : cs(_cs)
    {
        EnterCriticalSection(cs);
    }
    ~LockCritSect()
    {
        LeaveCriticalSection(cs);
    }
private:
    CRITICAL_SECTION *cs;
};

extern RootFilter rootFilter;